<?php
namespace nur\sery\wip\schema\_assoc;

use nulib\ref\schema\ref_analyze;
use nulib\ValueException;
use nur\sery\wip\schema\_scalar\ScalarResult;
use nur\sery\wip\schema\_scalar\ScalarWrapper;
use nur\sery\wip\schema\input\Input;
use nur\sery\wip\schema\Result;
use nur\sery\wip\schema\types\IType;
use nur\sery\wip\schema\Wrapper;
use nur\sery\wip\schema\WrapperContext;

class AssocWrapper extends Wrapper {
  function __construct(AssocSchema $schema, &$value=null, $valueKey=null, ?array $params=null) {
    $keys = $schema->getKeys();
    $keyWrappers = [];
    foreach ($keys as $key) {
      $keyWrappers[$key] = $schema->getSchema($key)->getWrapper();
    }
    $this->context = $context = new AssocWrapperContext($schema, null, null, $params);
    $context->arrayWrapper = new ScalarWrapper($schema, $dummy, null, null, $context);
    $context->keys = $keys;
    $context->keyWrappers = $keyWrappers;

    # calculer manuellemet throw ici parce que WrapperContext le met à true par
    # défaut. on veut pouvoir mettre temporairement throw à false si jamais il
    # n'est pas spécifié par l'utilisateur
    $throw = $params["throw"] ?? null;
    # Si $value est null, ne pas lancer d'exception, parce qu'on considère que
    # c'est une initialisation sans conséquences
    if ($throw === null && $value !== null) $throw = true;
    $context->throw = $throw ?? false;
    $this->reset($value, $valueKey);
    $context->throw = $throw ?? true;
  }

  function isAssoc(?AssocWrapper &$wrapper=null): bool { $wrapper = $this; return true; }

  /** @var AssocWrapperContext  */
  protected WrapperContext $context;

  protected function resetContext($resetSelectedKey): void {
    $context = $this->context;
    $context->arrayWrapper->getResult()->reset();
    foreach ($context->keyWrappers as $wrapper) {
      $wrapper->getResult()->reset();
    }
    $context->analyzed = false;
    $context->normalized = false;
    if ($resetSelectedKey) $context->selectedKey = null;
  }

  function reset(&$value, $valueKey=null, ?array $params=null): Wrapper {
    $context = $this->context;
    if ($value instanceof Input) $input = $value;
    else $input = $this->newInput($value);
    $context->input = $input;
    $context->valueKey = $valueKey;
    foreach ($context->keyWrappers as $key => $keyWrapper) {
      $keyInput = $input->addKey($valueKey);
      $keyWrapper->reset($keyInput, $key, ["analyze" => false]);
    }
    $this->afterModify($params, true);
    return $this;
  }

  function getKeys(): array {
    return $this->context->keys;
  }

  protected function _getWrapper($key): Wrapper {
    if ($key === null) return $this->context->arrayWrapper;
    $wrapper = $context->keyWrappers[$key] ?? null;
    if ($wrapper === null) throw ValueException::invalid_key($key);
    return $wrapper;
  }

  /** @param string|int|null $key */
  function select($key=null): Wrapper {
    $wrapper = $this->_getWrapper($key);
    $this->context->selectedKey = $key;
    return $wrapper;
  }

  /**
   * @param AssocWrapperContext $context
   * @param AssocWrapper $wrapper
   */
  static function _analyze(WrapperContext $context, Wrapper $wrapper, ?array $params): int {
    if ($context->ensureArray) {
      $valueKey = $context->valueKey;
      $array = $context->input->get($valueKey);
      if ($array === null) $context->input->set([], $valueKey);
    }
    $what = ScalarWrapper::_analyze($context, $wrapper, $params);
    /** @var ScalarResult $result */
    $result = $context->result;
    if (!$result->valid) return $what;
    foreach ($context->keyWrappers as $keyWrapper) {
      $keyWrapper->analyze($params);
      if (!$keyWrapper->isValid()) {
        $what = ref_analyze::INVALID;
        $result->addInvalidMessage($keyWrapper);
      }
    }
    return $what;
  }

  /**
   * @param AssocWrapperContext $context
   * @param AssocWrapper $wrapper
   */
  static function _normalize(WrapperContext $context, Wrapper $wrapper, ?array $params): bool {
    $modified = ScalarWrapper::_normalize($context, $wrapper, $params);
    foreach ($context->keyWrappers as $keyWrapper) {
      if ($keyWrapper->normalize($params)) $modified = true;
    }
    return $modified;
  }

  function getResult($key=false): Result {
    if ($key === false) $key = $this->context->selectedKey;
    return $this->_getWrapper($key)->getResult();
  }

  function getType($key=false): IType {
    if ($key === false) $key = $this->context->selectedKey;
    return $this->_getWrapper($key)->getType();
  }

  function get($default=null, $key=false) {
    $context = $this->context;
    if (!$context->arrayWrapper->isAvailable()) return $default;
    if ($key === false) $key = $context->selectedKey;
    return $this->_getWrapper($key)->get($default);
  }

  function set($value, ?array $params=null, $key=false): Wrapper {
    $context = $this->context;
    if ($key === false) $key = $context->selectedKey;
    $this->_getWrapper($key)->set($value);
    return $this;
  }

  function unset(?array $params=null, $key=false): Wrapper {
    $context = $this->context;
    if ($key === false) $key = $context->selectedKey;
    $this->_getWrapper($key)->unset();
    return $this;
  }

  function format($format=null, $key=false): string {
    $context = $this->context;
    if ($key === false) $key = $context->selectedKey;
    return $this->_getWrapper($key)->format($format);
  }
}
