<?php
namespace nur\sery\wip\schema\input;

use nulib\StateException;
use nur\sery\wip\php\access\IAccess;
use nur\sery\wip\php\access\KeyAccess;
use nur\sery\wip\php\access\PropertyAccess;

/**
 * Class Input: accès à une valeur
 *
 * cette implémentation lit depuis et écrit dans une référence
 */
class Input {
  const ALLOW_EMPTY = true;

  const ACCESS_AUTO = 0, ACCESS_KEY = 1, ACCESS_PROPERTY = 2;

  private static function unexpected_access_type(): StateException {
    return StateException::unexpected_state("access_type");
  }

  function __construct(&$dest=null, ?array $params=null) {
    $accessType = $params["access_type"] ?? self::ACCESS_AUTO;
    if ($accessType === self::ACCESS_AUTO) {
      $accessType = is_object($dest)? self::ACCESS_PROPERTY: self::ACCESS_KEY;
    }

    $allowEmpty = $params["allow_empty"] ?? static::ALLOW_EMPTY;
    if ($accessType == self::ACCESS_PROPERTY) {
      $this->access = new PropertyAccess($dest, null, [
        "allow_empty" => $allowEmpty,
        "allow_null" => true,
      ]);
    } elseif ($accessType == self::ACCESS_KEY) {
      $this->access = new KeyAccess($dest, null, [
        "allow_empty" => $allowEmpty,
        "allow_null" => true,
      ]);
    } else {
      throw self::unexpected_access_type();
    }
  }

  protected IAccess $access;

  /** tester si la valeur existe sans tenir compte de $allowEmpty */
  function isPresent($key=null): bool {
    return $this->access->resetKey($key)->exists();
  }

  /** tester si la valeur est disponible en tenant compte de $allowEmpty */
  function isAvailable($key=null): bool {
    return $this->access->resetKey($key)->available();
  }

  function get($key=null) {
    return $this->access->resetKey($key)->get();
  }

  function set($value, $key=null): void {
    $this->access->resetKey($key)->set($value);
  }

  function unset($key=null): void {
    $this->access->resetKey($key)->del();
  }

  function addKey($key): Input {
    if ($key === null) return $this;
    $input = clone $this;
    $input->access = $this->access->addKey($key);
    return $input;
  }
}
