<?php
namespace nur\sery\wip\php\access;

use nulib\tests\TestCase;
use stdClass;

class KeyAccessTest extends TestCase {
  function testValueAccess() {
    $default = new stdClass();

    #
    $i = null;
    $a = new KeyAccess($i);
    self::assertFalse($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $i = false;
    $a = new KeyAccess($i);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame(false, $a->get($default));

    $i = "";
    $a = new KeyAccess($i);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame("", $a->get($default));

    #
    $i = null;
    $a = new KeyAccess($i, null, ["allow_null" => false]);
    self::assertFalse($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $i = null;
    $a = new KeyAccess($i, null, ["allow_null" => true]);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame(null, $a->get($default));

    #
    $i = false;
    $a = new KeyAccess($i, null, ["allow_false" => false]);
    self::assertTrue($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $i = false;
    $a = new KeyAccess($i, null, ["allow_false" => true]);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame(false, $a->get($default));

    #
    $i = "";
    $a = new KeyAccess($i, null, ["allow_empty" => false]);
    self::assertTrue($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $i = "";
    $a = new KeyAccess($i, null, ["allow_empty" => true]);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame("", $a->get($default));
  }
  
  function testArrayAccess() {
    $default = new stdClass();
    $array = ["null" => null, "false" => false, "empty" => ""];

    #
    $a = new KeyAccess($array, "inexistant");
    self::assertFalse($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $a = new KeyAccess($array, "null");
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame(null, $a->get($default));

    $a = new KeyAccess($array, "false");
    self::assertTrue($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $a = new KeyAccess($array, "empty");
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame("", $a->get($default));

    #
    $a = new KeyAccess($array, "null", ["allow_null" => false]);
    self::assertTrue($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $a = new KeyAccess($array, "null", ["allow_null" => true]);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame(null, $a->get($default));

    #
    $a = new KeyAccess($array, "false", ["allow_false" => false]);
    self::assertTrue($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $a = new KeyAccess($array, "false", ["allow_false" => true]);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame(false, $a->get($default));

    #
    $a = new KeyAccess($array, "empty", ["allow_empty" => false]);
    self::assertTrue($a->exists());
    self::assertFalse($a->available());
    self::assertSame($default, $a->get($default));

    $a = new KeyAccess($array, "empty", ["allow_empty" => true]);
    self::assertTrue($a->exists());
    self::assertTrue($a->available());
    self::assertSame("", $a->get($default));
  }
}
